@@ -24,15 +24,17 @@
 #include "qgslayertreemodel.h"
 #include "qgslegendsettings.h"
 #include "qgslayertreegroup.h"
+#include "qgsexpressioncontext.h"
 
 class QgsLayerTreeModel;
 class QgsSymbol;
 class QgsLayoutItemMap;
 class QgsLegendRenderer;
+class QgsLayoutItemLegend;
 
 /**
  * \ingroup core
- * Item model implementation based on layer tree model for layout legend.
+ * \brief Item model implementation based on layer tree model for layout legend.
  *
  * Overrides some functionality of QgsLayerTreeModel to better fit the needs of layout legends.
  *
@@ -44,18 +46,68 @@ class CORE_EXPORT QgsLegendModel : public QgsLayerTreeModel
 
   public:
     //! Construct the model based on the given layer tree
-    QgsLegendModel( QgsLayerTree *rootNode, QObject *parent SIP_TRANSFERTHIS = nullptr );
+    QgsLegendModel( QgsLayerTree *rootNode, QObject *parent SIP_TRANSFERTHIS = nullptr, QgsLayoutItemLegend *layout = nullptr );
+
+    //! Alternative constructor.
+    QgsLegendModel( QgsLayerTree *rootNode,  QgsLayoutItemLegend *layout );
 
     QVariant data( const QModelIndex &index, int role ) const override;
 
     Qt::ItemFlags flags( const QModelIndex &index ) const override;
+
+    /**
+     * Returns filtered list of active legend nodes attached to a particular layer node
+     * (by default it returns also legend node embedded in parent layer node (if any) unless skipNodeEmbeddedInParent is TRUE)
+     * \note Parameter skipNodeEmbeddedInParent added in QGIS 2.18
+     * \note Not available in Python bindings
+     * \see layerOriginalLegendNodes()
+     * \since QGIS 3.10
+     */
+    QList<QgsLayerTreeModelLegendNode *> layerLegendNodes( QgsLayerTreeLayer *nodeLayer, bool skipNodeEmbeddedInParent = false ) const SIP_SKIP;
+
+    /**
+     * Clears any previously cached data for the specified \a node.
+     * \since QGIS 3.14
+     */
+    void clearCachedData( QgsLayerTreeNode *node ) const;
+
+  signals:
+
+    /**
+     * Emitted to refresh the legend.
+     * \since QGIS 3.10
+     */
+    void refreshLegend();
+
+  private slots:
+
+    /**
+     * Handle incoming signal to refresh the legend.
+     * \since QGIS 3.10
+     */
+    void forceRefresh();
+
+  private:
+
+    /**
+     * Pointer to the QgsLayoutItemLegend class that made the model.
+     * \since QGIS 3.10
+     */
+    QgsLayoutItemLegend *mLayoutLegend = nullptr;
+
+    /**
+     * Evaluate the expression or symbol expressions of a given layer node.
+     * \since QGIS 3.14
+     */
+    QString evaluateLayerExpressions( QgsLayerTreeLayer *nodeLayer ) const;
+
 };
 
 
 
 /**
  * \ingroup core
- * A layout item subclass for map legends.
+ * \brief A layout item subclass for map legends.
  * \since QGIS 3.0
  */
 class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
@@ -89,7 +141,7 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     /**
      * Sets whether the legend should automatically resize to fit its contents.
-     * \param enabled set to false to disable automatic resizing. The legend frame will not
+     * \param enabled set to FALSE to disable automatic resizing. The legend frame will not
      * be expanded to fit legend items, and items may be cropped from display.
      * \see resizeToContents()
      */
@@ -133,7 +185,7 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
     bool legendFilterByMapEnabled() const { return mLegendFilterByMap; }
 
     /**
-     * When set to true, during an atlas rendering, it will filter out legend elements
+     * When set to TRUE, during an atlas rendering, it will filter out legend elements
      * where features are outside the current atlas feature.
      * \see legendFilterOutAtlas()
      */
@@ -187,14 +239,18 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
     /**
      * Returns the font settings for a legend \a component.
      * \see setStyleFont()
+     *
+     * \deprecated use QgsLegendStyle::textFormat() from style() instead.
      */
-    QFont styleFont( QgsLegendStyle::Style component ) const;
+    Q_DECL_DEPRECATED QFont styleFont( QgsLegendStyle::Style component ) const SIP_DEPRECATED;
 
     /**
      * Sets the style \a font for a legend \a component.
      * \see styleFont()
+     *
+     * \deprecated use QgsLegendStyle::setTextFormat() from style() instead.
      */
-    void setStyleFont( QgsLegendStyle::Style component, const QFont &font );
+    Q_DECL_DEPRECATED void setStyleFont( QgsLegendStyle::Style component, const QFont &font ) SIP_DEPRECATED;
 
     /**
      * Set the \a margin for a legend \a component.
@@ -209,14 +265,18 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
     /**
      * Returns the spacing in-between lines in layout units.
      * \see setLineSpacing()
+     *
+     * \deprecated use QgsLegendStyle::textFormat() from style() instead.
      */
-    double lineSpacing() const;
+    Q_DECL_DEPRECATED double lineSpacing() const SIP_DEPRECATED;
 
     /**
      * Sets the \a spacing in-between multiple lines.
      * \see lineSpacing()
+     *
+     * \deprecated use QgsLegendStyle::setTextFormat() from style() instead.
      */
-    void setLineSpacing( double spacing );
+    Q_DECL_DEPRECATED void setLineSpacing( double spacing ) SIP_DEPRECATED;
 
     /**
      * Returns the legend box space.
@@ -245,14 +305,18 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
     /**
      * Returns the legend font color.
      * \see setFontColor()
+     *
+     * \deprecated use QgsLegendStyle::setTextFormat() from style() instead.
      */
-    QColor fontColor() const;
+    Q_DECL_DEPRECATED QColor fontColor() const SIP_DEPRECATED;
 
     /**
      * Sets the legend font \a color.
      * \see fontColor()
+     *
+     * \deprecated use QgsLegendStyle::setTextFormat() from style() instead.
      */
-    void setFontColor( const QColor &color );
+    Q_DECL_DEPRECATED void setFontColor( const QColor &color ) SIP_DEPRECATED;
 
     /**
      * Returns the legend symbol width.
@@ -266,6 +330,62 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
      */
     void setSymbolWidth( double width );
 
+    /**
+     * Returns the maximum symbol size (in mm). 0.0 means there is no maximum set.
+     *
+     * \see setMaximumSymbolSize()
+     * \since QGIS 3.16
+     */
+    double maximumSymbolSize() const;
+
+    /**
+     * Set the maximum symbol \a size for symbol (in millimeters).
+     *
+     * A symbol size of 0.0 indicates no maximum is set.
+     *
+     * \see maximumSymbolSize()
+     * \since QGIS 3.16
+     */
+    void setMaximumSymbolSize( double size );
+
+    /**
+     * Returns the minimum symbol size (in mm). A value 0.0 means there is no minimum set.
+     *
+     * \see setMinimumSymbolSize
+     * \since QGIS 3.16
+     */
+    double minimumSymbolSize() const;
+
+    /**
+     * Set the minimum symbol \a size for symbol (in millimeters).
+     *
+     * A symbol size of 0.0 indicates no minimum is set.
+     *
+     * \see minimumSymbolSize()
+     * \since QGIS 3.16
+     */
+    void setMinimumSymbolSize( double size );
+
+    /**
+     * Sets the \a alignment for placement of legend symbols.
+     *
+     * Only Qt::AlignLeft or Qt::AlignRight are supported values.
+     *
+     * \see symbolAlignment()
+     * \since QGIS 3.10
+     */
+    void setSymbolAlignment( Qt::AlignmentFlag alignment );
+
+    /**
+     * Returns the alignment for placement of legend symbols.
+     *
+     * Only Qt::AlignLeft or Qt::AlignRight are supported values.
+     *
+     * \see setSymbolAlignment()
+     * \since QGIS 3.10
+     */
+    Qt::AlignmentFlag symbolAlignment() const;
+
     /**
      * Returns the legend symbol height.
      * \see setSymbolHeight()
@@ -362,7 +482,7 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     /**
      * Sets whether a stroke will be drawn around raster symbol items.
-     * \param enabled set to true to draw borders
+     * \param enabled set to TRUE to draw borders
      * \see drawRasterStroke()
      * \see setRasterStrokeColor()
      * \see setRasterStrokeWidth()
@@ -371,7 +491,7 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     /**
      * Returns the stroke color for the stroke drawn around raster symbol items. The stroke is
-     * only drawn if drawRasterStroke() is true.
+     * only drawn if drawRasterStroke() is TRUE.
      * \see setRasterStrokeColor()
      * \see drawRasterStroke()
      * \see rasterStrokeWidth()
@@ -380,7 +500,7 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     /**
      * Sets the stroke \a color for the stroke drawn around raster symbol items. The stroke is
-     * only drawn if drawRasterStroke() is true.
+     * only drawn if drawRasterStroke() is TRUE.
      * \see rasterStrokeColor()
      * \see setDrawRasterStroke()
      * \see setRasterStrokeWidth()
@@ -389,7 +509,7 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     /**
      * Returns the stroke width (in layout units) for the stroke drawn around raster symbol items. The stroke is
-     * only drawn if drawRasterStroke() is true.
+     * only drawn if drawRasterStroke() is TRUE.
      * \see setRasterStrokeWidth()
      * \see drawRasterStroke()
      * \see rasterStrokeColor()
@@ -398,7 +518,7 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     /**
      * Sets the stroke width for the stroke drawn around raster symbol items. The stroke is
-     * only drawn if drawRasterStroke() is true.
+     * only drawn if drawRasterStroke() is TRUE.
      * \see rasterStrokeWidth()
      * \see setDrawRasterStroke()
      * \see setRasterStrokeColor()
@@ -417,6 +537,15 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
      */
     QgsLayoutItemMap *linkedMap() const { return mMap; }
 
+    /**
+     * Returns the name of the theme currently linked to the legend.
+     *
+     * This usually equates to the theme rendered in the linkedMap().
+     *
+     * \since QGIS 3.14
+     */
+    QString themeName() const;
+
     /**
      * Updates the model and all legend entries.
      */
@@ -437,11 +566,13 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
     void finalizeRestoreFromXml() override;
 
     QgsExpressionContext createExpressionContext() const override;
-
+    ExportLayerBehavior exportLayerBehavior() const override;
+    bool accept( QgsStyleEntityVisitorInterface *visitor ) const override;
 
   public slots:
 
     void refresh() override;
+    void invalidateCache() override;
     void refreshDataDefinedProperty( QgsLayoutObject::DataDefinedProperty property = QgsLayoutObject::AllProperties ) override;
 
   protected:
@@ -459,6 +590,8 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     //! update legend in case style of associated map has changed
     void mapLayerStyleOverridesChanged();
+    //! update legend in case theme of associated map has changed
+    void mapThemeChanged( const QString &theme );
 
     //! react to atlas
     void onAtlasEnded();
@@ -466,14 +599,19 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     void nodeCustomPropertyChanged( QgsLayerTreeNode *node, const QString &key );
 
+    //! Clears any data cached for the legend model
+    void clearLegendCachedData();
+
   private:
     QgsLayoutItemLegend() = delete;
 
-    //! use new custom layer tree and update model. if new root is null pointer, will use project's tree
+    //! use new custom layer tree and update model. if new root is NULLPTR, will use project's tree
     void setCustomLayerTree( QgsLayerTree *rootGroup );
 
     void setupMapConnections( QgsLayoutItemMap *map, bool connect = true );
 
+    void setModelStyleOverrides( const QMap<QString, QString> &overrides );
+
     std::unique_ptr< QgsLegendModel > mLegendModel;
     std::unique_ptr< QgsLayerTreeGroup > mCustomLayerTree;
 
@@ -498,15 +636,18 @@ class CORE_EXPORT QgsLayoutItemLegend : public QgsLayoutItem
 
     bool mInAtlas = false;
 
-    //! Will be false until the associated map scale and DPI have been calculated
+    //! Will be FALSE until the associated map scale and DPI have been calculated
     bool mInitialMapScaleCalculated = false;
 
-    //! Will be true if the legend size should be totally reset at next paint
+    //! Will be TRUE if the legend size should be totally reset at next paint
     bool mForceResize = false;
 
-    //! Will be true if the legend should be resized automatically to fit contents
+    //! Will be TRUE if the legend should be resized automatically to fit contents
     bool mSizeToContents = true;
 
+    //! Name of theme for legend -- usually the theme associated with the linked map.
+    QString mThemeName;
+
     friend class QgsCompositionConverter;
 
 };
