@@ -24,7 +24,8 @@
 #include "qgsfield.h"
 #include "qgsfeaturerequest.h"
 #include "qgsfeatureiterator.h"
-
+#include <unordered_set>
+#include <deque>
 #include <QCache>
 
 class QgsVectorLayer;
@@ -231,7 +232,7 @@ class CORE_EXPORT QgsVectorLayerCache : public QObject
      * \see isFidCached()
      * \since QGIS 3.0
      */
-    QgsFeatureIds cachedFeatureIds() const { return qgis::listToSet( mCache.keys() ); }
+    QgsFeatureIds cachedFeatureIds() const;
 
     /**
      * Gets the feature at the given feature id. Considers the changed, added, deleted and permanent features
@@ -267,7 +268,7 @@ class CORE_EXPORT QgsVectorLayerCache : public QObject
     /**
      * Returns the geometry type for features in the cache.
      */
-    QgsWkbTypes::Type wkbType() const;
+    Qgis::WkbType wkbType() const;
 
 #ifdef SIP_RUN
 
@@ -291,7 +292,7 @@ class CORE_EXPORT QgsVectorLayerCache : public QObject
      * Returns the number of features contained in the source, or -1
      * if the feature count is unknown.
      */
-    long featureCount() const;
+    long long featureCount() const;
 
   protected:
 
@@ -393,11 +394,22 @@ class CORE_EXPORT QgsVectorLayerCache : public QObject
     {
       QgsCachedFeature *cachedFeature = new QgsCachedFeature( feat, this );
       mCache.insert( feat.id(), cachedFeature );
+      if ( mCacheUnorderedKeys.find( feat.id() ) == mCacheUnorderedKeys.end() )
+      {
+        mCacheUnorderedKeys.insert( feat.id() );
+        mCacheOrderedKeys.emplace_back( feat.id() );
+      }
     }
 
     QgsVectorLayer *mLayer = nullptr;
     QCache< QgsFeatureId, QgsCachedFeature > mCache;
 
+    // we need two containers here. One is used for efficient tracking of the IDs which have been added to the cache, the other
+    // is used to store the order of the incoming feature ids, so that we can correctly iterate through features in the original order.
+    // the ordered list alone is far too slow to handle this -- searching for existing items in a list is magnitudes slower than the unordered_set
+    std::unordered_set< QgsFeatureId > mCacheUnorderedKeys;
+    std::deque< QgsFeatureId > mCacheOrderedKeys;
+
     bool mCacheGeometry = true;
     bool mFullCache = false;
     QList<QgsAbstractCacheIndex *> mCacheIndices;
