@@ -18,70 +18,41 @@ email                : lrssvtml (at) gmail (dot) com
  ***************************************************************************/
 Some portions of code were taken from https://code.google.com/p/pydee/
 """
-from qgis.PyQt.QtCore import Qt, QObject, QEvent, QCoreApplication, QFileInfo, QSize, QDir, QByteArray, QJsonDocument, QUrl
-from qgis.PyQt.QtGui import QFont, QColor, QKeySequence
-from qgis.PyQt.QtNetwork import QNetworkRequest
-from qgis.PyQt.QtWidgets import QShortcut, QMenu, QApplication, QWidget, QGridLayout, QSpacerItem, QSizePolicy, QFileDialog, QTabWidget, QTreeWidgetItem, QFrame, QLabel, QToolButton, QMessageBox
-from qgis.PyQt.Qsci import QsciScintilla, QsciStyle
-from qgis.core import (
-    Qgis,
-    QgsApplication,
-    QgsSettings,
-    QgsBlockingNetworkRequest,
-    QgsFileUtils
-)
-from qgis.gui import QgsMessageBar, QgsCodeEditorPython
-from qgis.utils import OverrideCursor
-import sys
-import os
-import subprocess
-import datetime
-import pyclbr
-from operator import itemgetter
-import traceback
+
 import codecs
-import re
 import importlib
+import os
+import pyclbr
+import re
+import sys
+import tempfile
 from functools import partial
+from operator import itemgetter
+from pathlib import Path
 
-
-class KeyFilter(QObject):
-    SHORTCUTS = {
-        ("Control", "T"): lambda w, t: w.newTabEditor(),
-        ("Control", "M"): lambda w, t: t.save(),
-        ("Control", "W"): lambda w, t: t.close()
-    }
-
-    def __init__(self, window, tab, *args):
-        QObject.__init__(self, *args)
-        self.window = window
-        self.tab = tab
-        self._handlers = {}
-        for shortcut, handler in list(KeyFilter.SHORTCUTS.items()):
-            modifiers = shortcut[0]
-            if not isinstance(modifiers, list):
-                modifiers = [modifiers]
-            qt_mod_code = Qt.NoModifier
-            for each in modifiers:
-                qt_mod_code |= getattr(Qt, each + "Modifier")
-            qt_keycode = getattr(Qt, "Key_" + shortcut[1].upper())
-            handlers = self._handlers.get(qt_keycode, [])
-            handlers.append((qt_mod_code, handler))
-            self._handlers[qt_keycode] = handlers
-
-    def get_handler(self, key, modifier):
-        if self.window.count() > 1:
-            for modifiers, handler in self._handlers.get(key, []):
-                if modifiers == modifier:
-                    return handler
-        return None
-
-    def eventFilter(self, obj, event):
-        if event.type() == QEvent.KeyPress and event.key() < 256:
-            handler = self.get_handler(event.key(), event.modifiers())
-            if handler:
-                handler(self.window, self.tab)
-        return QObject.eventFilter(self, obj, event)
+from qgis.core import Qgis, QgsApplication, QgsBlockingNetworkRequest, QgsFileUtils, QgsSettings
+from qgis.gui import QgsCodeEditorPython, QgsMessageBar
+from qgis.PyQt.Qsci import QsciScintilla
+from qgis.PyQt.QtCore import QByteArray, QCoreApplication, QDir, QEvent, QFileInfo, QJsonDocument, QSize, Qt, QUrl
+from qgis.PyQt.QtGui import QKeySequence
+from qgis.PyQt.QtNetwork import QNetworkRequest
+from qgis.PyQt.QtWidgets import (
+    QApplication,
+    QFileDialog,
+    QFrame,
+    QGridLayout,
+    QLabel,
+    QMenu,
+    QMessageBox,
+    QShortcut,
+    QSizePolicy,
+    QSpacerItem,
+    QTabWidget,
+    QToolButton,
+    QTreeWidgetItem,
+    QWidget,
+)
+from qgis.utils import OverrideCursor
 
 
 class Editor(QgsCodeEditorPython):
@@ -89,10 +60,9 @@ class Editor(QgsCodeEditorPython):
     def __init__(self, parent=None):
         super().__init__(parent)
         self.parent = parent
+        self.path = None
         #  recent modification time
         self.lastModified = 0
-        self.opening = ['(', '{', '[', "'", '"']
-        self.closing = [')', '}', ']', "'", '"']
         self.settings = QgsSettings()
 
         self.setMinimumHeight(120)
@@ -122,12 +92,6 @@ class Editor(QgsCodeEditorPython):
         self.syntaxCheckScut = QShortcut(QKeySequence(Qt.CTRL + Qt.Key_4), self)
         self.syntaxCheckScut.setContext(Qt.WidgetShortcut)
         self.syntaxCheckScut.activated.connect(self.syntaxCheck)
-        self.commentScut = QShortcut(QKeySequence(Qt.CTRL + Qt.Key_3), self)
-        self.commentScut.setContext(Qt.WidgetShortcut)
-        self.commentScut.activated.connect(self.parent.pc.commentCode)
-        self.uncommentScut = QShortcut(QKeySequence(Qt.SHIFT + Qt.CTRL + Qt.Key_3), self)
-        self.uncommentScut.setContext(Qt.WidgetShortcut)
-        self.uncommentScut.activated.connect(self.parent.pc.uncommentCode)
         self.modificationChanged.connect(self.parent.modified)
         self.modificationAttempted.connect(self.fileReadOnly)
 
@@ -152,7 +116,7 @@ class Editor(QgsCodeEditorPython):
                                           self.searchSelectedTextInPyQGISDocs)
         menu.addAction(QgsApplication.getThemeIcon("mActionStart.svg"),
                        QCoreApplication.translate("PythonConsole", "Run Script"),
-                       self.runScriptCode, 'Shift+Ctrl+E')
+                       self.runScriptCode, 'Ctrl+Shift+E')
         menu.addSeparator()
         undoAction = menu.addAction(QgsApplication.getThemeIcon("mActionUndo.svg"),
                                     QCoreApplication.translate("PythonConsole", "Undo"),
@@ -178,11 +142,8 @@ class Editor(QgsCodeEditorPython):
             self.selectAll, QKeySequence.SelectAll)
         menu.addSeparator()
         menu.addAction(QgsApplication.getThemeIcon("console/iconCommentEditorConsole.svg"),
-                       QCoreApplication.translate("PythonConsole", "Comment"),
-                       self.parent.pc.commentCode, 'Ctrl+3')
-        menu.addAction(QgsApplication.getThemeIcon("console/iconUncommentEditorConsole.svg"),
-                       QCoreApplication.translate("PythonConsole", "Uncomment"),
-                       self.parent.pc.uncommentCode, 'Shift+Ctrl+3')
+                       QCoreApplication.translate("PythonConsole", "Toggle Comment"),
+                       self.toggleComment, 'Ctrl+:')
         menu.addSeparator()
         gist_menu = QMenu(self)
         gist_menu.setTitle(QCoreApplication.translate("PythonConsole", "Share on GitHub"))
@@ -198,7 +159,7 @@ class Editor(QgsCodeEditorPython):
         menu.addSeparator()
         menu.addAction(QgsApplication.getThemeIcon("console/iconSettingsConsole.svg"),
                        QCoreApplication.translate("PythonConsole", "Options…"),
-                       self.parent.pc.openSettings)
+                       self.pythonconsole.openSettings)
         syntaxCheckAction.setEnabled(False)
         pasteAction.setEnabled(False)
         pyQGISHelpAction.setEnabled(False)
@@ -237,11 +198,11 @@ class Editor(QgsCodeEditorPython):
             index = 0
         else:
             line, index = self.getCursorPosition()
-        text = self.parent.pc.lineEditFind.text()
+        text = self.pythonconsole.lineEditFind.text()
         re = False
-        wrap = self.parent.pc.wrapAround.isChecked()
-        cs = self.parent.pc.caseSensitive.isChecked()
-        wo = self.parent.pc.wholeWord.isChecked()
+        wrap = self.pythonconsole.wrapAround.isChecked()
+        cs = self.pythonconsole.caseSensitive.isChecked()
+        wo = self.pythonconsole.wholeWord.isChecked()
         notFound = False
         if text:
             if not forward:
@@ -257,10 +218,10 @@ class Editor(QgsCodeEditorPython):
                 if showMessage:
                     msgText = QCoreApplication.translate('PythonConsole',
                                                          '<b>"{0}"</b> was not found.').format(text)
-                    self.parent.pc.callWidgetMessageBarEditor(msgText, 0, True)
+                    self.pythonconsole.callWidgetMessageBarEditor(msgText, 0, True)
             else:
                 styleError = ''
-            self.parent.pc.lineEditFind.setStyleSheet(styleError)
+            self.pythonconsole.lineEditFind.setStyleSheet(styleError)
 
     def findNext(self):
         self.findText(True)
@@ -269,25 +230,25 @@ class Editor(QgsCodeEditorPython):
         self.findText(False)
 
     def objectListEditor(self):
-        listObj = self.parent.pc.listClassMethod
+        listObj = self.pythonconsole.listClassMethod
         if listObj.isVisible():
             listObj.hide()
-            self.parent.pc.objectListButton.setChecked(False)
+            self.pythonconsole.objectListButton.setChecked(False)
         else:
             listObj.show()
-            self.parent.pc.objectListButton.setChecked(True)
+            self.pythonconsole.objectListButton.setChecked(True)
 
     def shareOnGist(self, is_public):
         ACCESS_TOKEN = self.settings.value("pythonConsole/accessTokenGithub", '', type=QByteArray)
         if not ACCESS_TOKEN:
             msg_text = QCoreApplication.translate(
                 'PythonConsole', 'GitHub personal access token must be generated (see Console Options)')
-            self.parent.pc.callWidgetMessageBarEditor(msg_text, 1, True)
+            self.pythonconsole.callWidgetMessageBarEditor(msg_text, 1, True)
             return
 
         URL = "https://api.github.com/gists"
 
-        path = self.parent.tw.currentWidget().path
+        path = self.tabwidget.currentWidget().path
         filename = os.path.basename(path) if path else None
         filename = filename if filename else "pyqgis_snippet.py"
 
@@ -309,154 +270,71 @@ class Editor(QgsCodeEditorPython):
             link = _json.object()['html_url'].toString()
             QApplication.clipboard().setText(link)
             msg = QCoreApplication.translate('PythonConsole', 'URL copied to clipboard.')
-            self.parent.pc.callWidgetMessageBarEditor(msg, 0, True)
+            self.pythonconsole.callWidgetMessageBarEditor(msg, 0, True)
         else:
             msg = QCoreApplication.translate('PythonConsole', 'Connection error: ')
-            self.parent.pc.callWidgetMessageBarEditor(msg + request.erroMessage(), 0, True)
+            self.pythonconsole.callWidgetMessageBarEditor(msg + request.erroMessage(), 0, True)
 
     def hideEditor(self):
-        self.parent.pc.splitterObj.hide()
-        self.parent.pc.showEditorButton.setChecked(False)
+        self.pythonconsole.splitterObj.hide()
+        self.pythonconsole.showEditorButton.setChecked(False)
 
     def openFindWidget(self):
-        wF = self.parent.pc.widgetFind
+        wF = self.pythonconsole.widgetFind
         wF.show()
         if self.hasSelectedText():
-            self.parent.pc.lineEditFind.setText(self.selectedText().strip())
-        self.parent.pc.lineEditFind.setFocus()
-        self.parent.pc.findTextButton.setChecked(True)
+            self.pythonconsole.lineEditFind.setText(self.selectedText().strip())
+        self.pythonconsole.lineEditFind.setFocus()
+        self.pythonconsole.findTextButton.setChecked(True)
 
     def closeFindWidget(self):
-        wF = self.parent.pc.widgetFind
+        wF = self.pythonconsole.widgetFind
         wF.hide()
-        self.parent.pc.findTextButton.setChecked(False)
+        self.pythonconsole.findTextButton.setChecked(False)
 
     def toggleFindWidget(self):
-        wF = self.parent.pc.widgetFind
+        wF = self.pythonconsole.widgetFind
         if wF.isVisible():
             self.closeFindWidget()
         else:
             self.openFindWidget()
 
-    def commentEditorCode(self, commentCheck):
-        self.beginUndoAction()
-        if self.hasSelectedText():
-            startLine, _, endLine, _ = self.getSelection()
-            for line in range(startLine, endLine + 1):
-                if commentCheck:
-                    self.insertAt('#', line, 0)
-                else:
-                    if not self.text(line).strip().startswith('#'):
-                        continue
-                    self.setSelection(line, self.indentation(line),
-                                      line, self.indentation(line) + 1)
-                    self.removeSelectedText()
-        else:
-            line, pos = self.getCursorPosition()
-            if commentCheck:
-                self.insertAt('#', line, 0)
-            else:
-                if not self.text(line).strip().startswith('#'):
-                    return
-                self.setSelection(line, self.indentation(line),
-                                  line, self.indentation(line) + 1)
-                self.removeSelectedText()
-        self.endUndoAction()
-
     def createTempFile(self):
-        import tempfile
-        fd, path = tempfile.mkstemp()
-        tmpFileName = path + '.py'
-        with codecs.open(path, "w", encoding='utf-8') as f:
+        name = tempfile.NamedTemporaryFile(delete=False).name
+        # Need to use newline='' to avoid adding extra \r characters on Windows
+        with open(name, 'w', encoding='utf-8', newline='') as f:
             f.write(self.text())
-        os.close(fd)
-        os.rename(path, tmpFileName)
-        return tmpFileName
-
-    def _runSubProcess(self, filename, tmp=False):
-        dir = QFileInfo(filename).path()
-        file = QFileInfo(filename).fileName()
-        name = QFileInfo(filename).baseName()
-        if dir not in sys.path:
-            sys.path.append(dir)
-        if name in sys.modules:
-            importlib.reload(sys.modules[name])  # NOQA
-        try:
-            # set creationflags for running command without shell window
-            if sys.platform.startswith('win'):
-                p = subprocess.Popen(['python3', filename], shell=False, stdin=subprocess.PIPE,
-                                     stderr=subprocess.PIPE, stdout=subprocess.PIPE, creationflags=0x08000000)
-            else:
-                p = subprocess.Popen(['python3', filename], shell=False, stdin=subprocess.PIPE,
-                                     stderr=subprocess.PIPE, stdout=subprocess.PIPE)
-            out, _traceback = p.communicate()
-
-            # Fix interrupted system call on OSX
-            if sys.platform == 'darwin':
-                status = None
-                while status is None:
-                    try:
-                        status = p.wait()
-                    except OSError as e:
-                        if e.errno == 4:
-                            pass
-                        else:
-                            raise e
-            if tmp:
-                tmpFileTr = QCoreApplication.translate('PythonConsole', ' [Temporary file saved in {0}]').format(dir)
-                file = file + tmpFileTr
-            if _traceback:
-                msgTraceTr = QCoreApplication.translate('PythonConsole', '## Script error: {0}').format(file)
-                print("## {}".format(datetime.datetime.now()))
-                print(msgTraceTr)
-                sys.stderr.write(_traceback)
-                p.stderr.close()
-            else:
-                msgSuccessTr = QCoreApplication.translate('PythonConsole',
-                                                          '## Script executed successfully: {0}').format(file)
-                print("## {}".format(datetime.datetime.now()))
-                print(msgSuccessTr)
-                sys.stdout.write(out)
-                p.stdout.close()
-            del p
-            if tmp:
-                os.remove(filename)
-        except IOError as error:
-            IOErrorTr = QCoreApplication.translate('PythonConsole',
-                                                   'Cannot execute file {0}. Error: {1}\n').format(filename,
-                                                                                                   error.strerror)
-            print('## Error: ' + IOErrorTr)
-        except:
-            s = traceback.format_exc()
-            print('## Error: ')
-            sys.stderr.write(s)
+        return name
 
     def runScriptCode(self):
         autoSave = self.settings.value("pythonConsole/autoSaveScript", False, type=bool)
-        tabWidget = self.parent.tw.currentWidget()
+        tabWidget = self.tabwidget.currentWidget()
         filename = tabWidget.path
         msgEditorBlank = QCoreApplication.translate('PythonConsole',
                                                     'Hey, type something to run!')
         if filename is None:
             if not self.isModified():
-                self.parent.pc.callWidgetMessageBarEditor(msgEditorBlank, 0, True)
+                self.pythonconsole.callWidgetMessageBarEditor(msgEditorBlank, 0, True)
                 return
 
+        deleteTempFile = False
         if self.syntaxCheck():
             if filename and self.isModified() and autoSave:
-                self.parent.save(filename)
+                self.save(filename)
             elif not filename or self.isModified():
                 # Create a new temp file if the file isn't already saved.
-                tmpFile = self.createTempFile()
-                filename = tmpFile
+                filename = self.createTempFile()
+                deleteTempFile = True
 
-            self.parent.pc.shell.runCommand("exec(Path('{0}').read_text())"
-                                            .format(filename.replace("\\", "/")))
+            self.pythonconsole.shell.runCommand("exec(Path('{0}').read_text())"
+                                                .format(filename.replace("\\", "/")))
+            if deleteTempFile:
+                Path(filename).unlink()
 
     def runSelectedCode(self):  # spellok
         cmd = self.selectedText()
-        self.parent.pc.shell.insertFromDropPaste(cmd)
-        self.parent.pc.shell.entered()
+        self.pythonconsole.shell.insertFromDropPaste(cmd)
+        self.pythonconsole.shell.entered()
         self.setFocus()
 
     def getTextFromEditor(self):
@@ -477,25 +355,15 @@ class Editor(QgsCodeEditorPython):
         self.setFocus()
 
     def syntaxCheck(self):
-        source = self.text()
         self.clearWarnings()
+        source = self.text().encode("utf-8")
         try:
-            filename = self.parent.tw.currentWidget().path
-            if not filename:
-                tmpFile = self.createTempFile()
-                filename = tmpFile
-            if isinstance(source, type("")):
-                source = source.encode('utf-8')
-            if isinstance(filename, type("")):
-                filename = filename.encode('utf-8')
-            if filename:
-                compile(source, filename, 'exec')
+            compile(source, "", "exec")
         except SyntaxError as detail:
-            eline = detail.lineno and detail.lineno or 1
+            eline = detail.lineno or 1
             eline -= 1
-            ecolumn = detail.offset and detail.offset or 1
+            ecolumn = detail.offset or 1
             edescr = detail.msg
-
             self.addWarning(eline, edescr)
             self.setCursorPosition(eline, ecolumn - 1)
             self.ensureLineVisible(eline)
@@ -503,105 +371,138 @@ class Editor(QgsCodeEditorPython):
 
         return True
 
-    def keyPressEvent(self, e):
-        t = e.text()
-        startLine, _, endLine, endPos = self.getSelection()
-        line, pos = self.getCursorPosition()
-        self.autoCloseBracket = self.settings.value("pythonConsole/autoCloseBracket", False, type=bool)
-        self.autoImport = self.settings.value("pythonConsole/autoInsertionImport", True, type=bool)
-        txt = self.text(line)[:pos]
-        # Close bracket automatically
-        if t in self.opening and self.autoCloseBracket:
-            self.beginUndoAction()
-            i = self.opening.index(t)
-            if self.hasSelectedText():
-                selText = self.selectedText()
-                self.removeSelectedText()
-                if startLine == endLine:
-                    self.insert(self.opening[i] + selText + self.closing[i])
-                    self.setCursorPosition(endLine, endPos + 2)
-                    self.endUndoAction()
-                    return
-                elif startLine < endLine and self.opening[i] in ("'", '"'):
-                    self.insert("'''" + selText + "'''")
-                    self.setCursorPosition(endLine, endPos + 3)
-                    self.endUndoAction()
-                    return
-            elif t == '(' and (re.match(r'^[ \t]*def \w+$', txt) or re.match(r'^[ \t]*class \w+$', txt)):
-                self.insert('):')
-            else:
-                self.insert(self.closing[i])
-            self.endUndoAction()
-        # FIXES #8392 (automatically removes the redundant char
-        # when autoclosing brackets option is enabled)
-        elif t in [')', ']', '}'] and self.autoCloseBracket:
-            txt = self.text(line)
-            try:
-                if txt[pos - 1] in self.opening and t == txt[pos]:
-                    self.setCursorPosition(line, pos + 1)
-                    self.SendScintilla(QsciScintilla.SCI_DELETEBACK)
-            except IndexError:
-                pass
-        elif t == ' ' and self.autoImport:
-            ptrn = r'^[ \t]*from [\w.]+$'
-            if re.match(ptrn, txt):
-                self.insert(' import')
-                self.setCursorPosition(line, pos + 7)
-        QsciScintilla.keyPressEvent(self, e)
-
     def focusInEvent(self, e):
-        pathfile = self.parent.path
-        if pathfile:
-            if not QFileInfo(pathfile).exists():
+        if self.path:
+            if not QFileInfo(self.path).exists():
                 msgText = QCoreApplication.translate('PythonConsole',
-                                                     'The file <b>"{0}"</b> has been deleted or is not accessible').format(pathfile)
-                self.parent.pc.callWidgetMessageBarEditor(msgText, 2, False)
+                                                     'The file <b>"{0}"</b> has been deleted or is not accessible').format(self.path)
+                self.pythonconsole.callWidgetMessageBarEditor(msgText, 2, False)
                 return
-        if pathfile and self.lastModified != QFileInfo(pathfile).lastModified():
+        if self.path and self.lastModified != QFileInfo(self.path).lastModified():
             self.beginUndoAction()
             self.selectAll()
-            # fileReplaced = self.selectedText()
             self.removeSelectedText()
-            file = open(pathfile, "r")
-            fileLines = file.readlines()
-            file.close()
-            with OverrideCursor(Qt.WaitCursor):
-                for line in reversed(fileLines):
-                    self.insert(line)
+            self.insert(Path(self.path).read_text(encoding='utf-8'))
             self.setModified(False)
             self.endUndoAction()
 
-            self.parent.tw.listObject(self.parent.tw.currentWidget())
-            self.lastModified = QFileInfo(pathfile).lastModified()
-        QsciScintilla.focusInEvent(self, e)
+            self.tabwidget.listObject(self.tabwidget.currentWidget())
+            self.lastModified = QFileInfo(self.path).lastModified()
+        super().focusInEvent(e)
 
     def fileReadOnly(self):
-        tabWidget = self.parent.tw.currentWidget()
+        tabWidget = self.tabwidget.currentWidget()
         msgText = QCoreApplication.translate('PythonConsole',
                                              'The file <b>"{0}"</b> is read only, please save to different file first.').format(tabWidget.path)
-        self.parent.pc.callWidgetMessageBarEditor(msgText, 1, False)
+        self.pythonconsole.callWidgetMessageBarEditor(msgText, 1, False)
+
+    def loadFile(self, filename, readOnly=False):
+        self.lastModified = QFileInfo(filename).lastModified()
+        self.path = filename
+        self.setText(Path(filename).read_text(encoding='utf-8'))
+        self.setReadOnly(readOnly)
+        self.setModified(False)
+        self.recolor()
+
+    def save(self, filename=None):
+        if self.isReadOnly():
+            return
+        tabwidget = self.tabwidget
+        index = tabwidget.indexOf(self.parent)
+        if filename:
+            self.path = filename
+        if self.path is None:
+            saveTr = QCoreApplication.translate('PythonConsole',
+                                                'Python Console: Save file')
+            folder = self.pythonconsole.settings.value("pythonConsole/lastDirPath", QDir.homePath())
+            self.path, filter = QFileDialog().getSaveFileName(self,
+                                                              saveTr,
+                                                              os.path.join(folder, tabwidget.tabText(index).replace('*', '') + '.py'),
+                                                              "Script file (*.py)")
+            # If the user didn't select a file, abort the save operation
+            if not self.path:
+                self.path = None
+                return
+
+            msgText = QCoreApplication.translate('PythonConsole',
+                                                 'Script was correctly saved.')
+            self.pythonconsole.callWidgetMessageBarEditor(msgText, 0, True)
+
+        # Save the new contents
+        # Need to use newline='' to avoid adding extra \r characters on Windows
+        with open(self.path, 'w', encoding='utf-8', newline='') as f:
+            f.write(self.text())
+        tabwidget.setTabTitle(index, Path(self.path).name)
+        tabwidget.setTabToolTip(index, self.path)
+        self.setModified(False)
+        self.pythonconsole.saveFileButton.setEnabled(False)
+        self.lastModified = QFileInfo(self.path).lastModified()
+        self.pythonconsole.updateTabListScript(self.path, action='append')
+        tabwidget.listObject(self.parent)
+        lastDirPath = str(Path(self.path).parent)
+        self.pythonconsole.settings.setValue("pythonConsole/lastDirPath", lastDirPath)
+
+    def event(self, e):
+        """ Used to override the Application shortcuts when the editor has focus """
+
+        if e.type() == QEvent.ShortcutOverride:
+            ctrl = e.modifiers() == Qt.ControlModifier
+            ctrl_shift = e.modifiers() == (Qt.ControlModifier | Qt.ShiftModifier)
+            if (
+                (ctrl and e.key() == Qt.Key_W)
+                or (ctrl_shift and e.key() == Qt.Key_W)
+                or (ctrl and e.key() == Qt.Key_S)
+                or (ctrl_shift and e.key() == Qt.Key_S)
+                or (ctrl and e.key() == Qt.Key_T)
+                or (ctrl and e.key() == Qt.Key_Tab)
+            ):
+                e.accept()
+                return True
+
+        return super().event(e)
+
+    def keyPressEvent(self, e):
+        ctrl = e.modifiers() == Qt.ControlModifier
+        ctrl_shift = e.modifiers() == (Qt.ControlModifier | Qt.ShiftModifier)
+
+        # Ctrl+W: close current tab
+        if ctrl and e.key() == Qt.Key_W:
+            self.parent.close()
+
+        # Ctrl+Shift+W: close all tabs
+        if ctrl_shift and e.key() == Qt.Key_W:
+            self.tabwidget.closeAll()
+
+        # Ctrl+S: save current tab
+        if ctrl and e.key() == Qt.Key_S:
+            self.save()
+
+        # Ctrl+Shift+S: save current tab as
+        if ctrl_shift and e.key() == Qt.Key_S:
+            self.tabwidget.saveAs()
+
+        # Ctrl+T: open new tab
+        if ctrl and e.key() == Qt.Key_T:
+            self.tabwidget.newTabEditor()
+
+        super().keyPressEvent(e)
 
 
 class EditorTab(QWidget):
 
-    def __init__(self, parent, parentConsole, filename, readOnly):
-        super(EditorTab, self).__init__(parent)
-        self.tw = parent
-        self.pc = parentConsole
-        self.path = None
-        self.readOnly = readOnly
+    def __init__(self, parent, pythonconsole, filename, readOnly):
+        super().__init__(parent)
+        self.tabwidget = parent
 
-        self.fileExecuteList = {}
-        self.fileExecuteList = dict()
-
-        self.newEditor = Editor(self)
+        self._editor = Editor(self)
+        self._editor.pythonconsole = pythonconsole
+        self._editor.tabwidget = parent
         if filename:
-            self.path = filename
             if QFileInfo(filename).exists():
-                self.loadFile(filename, False)
+                self._editor.loadFile(filename, readOnly)
 
         # Creates layout for message bar
-        self.layout = QGridLayout(self.newEditor)
+        self.layout = QGridLayout(self._editor)
         self.layout.setContentsMargins(0, 0, 0, 0)
         spacerItem = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
         self.layout.addItem(spacerItem, 1, 0, 1, 1)
@@ -613,95 +514,33 @@ class EditorTab(QWidget):
 
         self.tabLayout = QGridLayout(self)
         self.tabLayout.setContentsMargins(0, 0, 0, 0)
-        self.tabLayout.addWidget(self.newEditor)
-
-        self.keyFilter = KeyFilter(parent, self)
-        self.setEventFilter(self.keyFilter)
-
-    def loadFile(self, filename, modified):
-        self.newEditor.lastModified = QFileInfo(filename).lastModified()
-        fn = codecs.open(filename, "rb", encoding='utf-8')
-        txt = fn.read()
-        fn.close()
-        with OverrideCursor(Qt.WaitCursor):
-            self.newEditor.setText(txt)
-            if self.readOnly:
-                self.newEditor.setReadOnly(self.readOnly)
-        self.newEditor.setModified(modified)
-        self.newEditor.recolor()
-
-    def save(self, fileName=None):
-        index = self.tw.indexOf(self)
-        if fileName:
-            self.path = fileName
-        if self.path is None:
-            saveTr = QCoreApplication.translate('PythonConsole',
-                                                'Python Console: Save file')
-            folder = self.pc.settings.value("pythonConsole/lastDirPath", QDir.homePath())
-            self.path, filter = QFileDialog().getSaveFileName(self,
-                                                              saveTr,
-                                                              os.path.join(folder, self.tw.tabText(index).replace('*', '') + '.py'),
-                                                              "Script file (*.py)")
-            # If the user didn't select a file, abort the save operation
-            if len(self.path) == 0:
-                self.path = None
-                return
-
-            self.path = QgsFileUtils.ensureFileNameHasExtension(self.path, ['py'])
-            self.tw.setCurrentWidget(self)
-            msgText = QCoreApplication.translate('PythonConsole',
-                                                 'Script was correctly saved.')
-            self.pc.callWidgetMessageBarEditor(msgText, 0, True)
-        # Rename the original file, if it exists
-        path = self.path
-        overwrite = QFileInfo(path).exists()
-        if overwrite:
-            try:
-                permis = os.stat(path).st_mode
-                # self.newEditor.lastModified = QFileInfo(path).lastModified()
-                os.chmod(path, permis)
-            except:
-                raise
-
-            temp_path = path + "~"
-            if QFileInfo(temp_path).exists():
-                os.remove(temp_path)
-            os.rename(path, temp_path)
-        # Save the new contents
-        with codecs.open(path, "w", encoding='utf-8') as f:
-            f.write(self.newEditor.text())
-        if overwrite:
-            os.remove(temp_path)
-        if self.newEditor.isReadOnly():
-            self.newEditor.setReadOnly(False)
-        fN = path.split('/')[-1]
-        self.tw.setTabTitle(index, fN)
-        self.tw.setTabToolTip(index, path)
-        self.newEditor.setModified(False)
-        self.pc.saveFileButton.setEnabled(False)
-        self.newEditor.lastModified = QFileInfo(path).lastModified()
-        self.pc.updateTabListScript(path, action='append')
-        self.tw.listObject(self)
-        lastDirPath = QFileInfo(path).path()
-        self.pc.settings.setValue("pythonConsole/lastDirPath", lastDirPath)
+        self.tabLayout.addWidget(self._editor)
 
     def modified(self, modified):
-        self.tw.tabModified(self, modified)
+        self.tabwidget.tabModified(self, modified)
 
     def close(self):
-        self.tw._removeTab(self, tab2index=True)
+        self.tabwidget._removeTab(self, tab2index=True)
 
-    def setEventFilter(self, filter):
-        self.newEditor.installEventFilter(filter)
+    def __getattr__(self, name):
+        """ Forward all missing attribute requests to the editor."""
+        try:
+            return super().__getattr__(name)
+        except AttributeError:
+            return getattr(self._editor, name)
 
-    def newTab(self):
-        self.tw.newTabEditor()
+    def __setattr__(self, name, value):
+        """ Forward all missing attribute requests to the editor."""
+        try:
+            return super().__setattr__(name, value)
+        except AttributeError:
+            return setattr(self._editor, name, value)
 
 
 class EditorTabWidget(QTabWidget):
 
     def __init__(self, parent):
-        QTabWidget.__init__(self, parent=None)
+        super().__init__(parent=None)
         self.parent = parent
 
         self.settings = QgsSettings()
@@ -833,7 +672,7 @@ class EditorTabWidget(QTabWidget):
                 closeTabAction.setEnabled(True)
                 closeAllTabAction.setEnabled(True)
                 closeOthersTabAction.setEnabled(True)
-            if self.widget(self.idx).newEditor.isModified():
+            if self.widget(self.idx).isModified():
                 saveAction.setEnabled(True)
             menu.exec_(self.mapToGlobal(e.pos()))
 
@@ -847,18 +686,17 @@ class EditorTabWidget(QTabWidget):
         countTab = self.count()
         for i in range(countTab - 1, 0, -1):
             self._removeTab(i)
-        self.newTabEditor(tabName='Untitled-0')
+
+        self.newTabEditor(tabName=QCoreApplication.translate("PythonConsole", "Untitled-0"))
         self._removeTab(0)
 
     def saveAs(self):
-        idx = self.idx
-        self.parent.saveAsScriptFile(idx)
-        self.setCurrentWidget(self.widget(idx))
+        self.parent.saveAsScriptFile(self.idx)
 
     def enableSaveIfModified(self, tab):
         tabWidget = self.widget(tab)
         if tabWidget:
-            self.parent.saveFileButton.setEnabled(tabWidget.newEditor.isModified())
+            self.parent.saveFileButton.setEnabled(tabWidget.isModified())
 
     def enableToolBarEditor(self, enable):
         if self.topFrame.isVisible():
@@ -884,10 +722,10 @@ class EditorTabWidget(QTabWidget):
         nr = self.count()
         if not tabName:
             tabName = QCoreApplication.translate('PythonConsole', 'Untitled-{0}').format(nr)
-        self.tab = EditorTab(self, self.parent, filename, readOnly)
+        tab = EditorTab(self, self.parent, filename, readOnly)
         self.iconTab = QgsApplication.getThemeIcon('console/iconTabEditorConsole.svg')
-        self.addTab(self.tab, self.iconTab, tabName + ' (ro)' if readOnly else tabName)
-        self.setCurrentWidget(self.tab)
+        self.addTab(tab, self.iconTab, tabName + ' (ro)' if readOnly else tabName)
+        self.setCurrentWidget(tab)
         if filename:
             self.setTabToolTip(self.currentIndex(), filename)
         else:
@@ -899,22 +737,14 @@ class EditorTabWidget(QTabWidget):
         self.setTabTitle(index, '*{}'.format(s) if modified else re.sub(r'^(\*)', '', s))
         self.parent.saveFileButton.setEnabled(modified)
 
-    def closeTab(self, tab):
-        if self.count() < 2:
-            self.removeTab(self.indexOf(tab))
-            self.newTabEditor()
-        else:
-            self.removeTab(self.indexOf(tab))
-        self.currentWidget().setFocus(Qt.TabFocusReason)
-
     def setTabTitle(self, tab, title):
         self.setTabText(tab, title)
 
     def _removeTab(self, tab, tab2index=False):
         if tab2index:
             tab = self.indexOf(tab)
-        tabWidget = self.widget(tab)
-        if tabWidget.newEditor.isModified():
+        editorTab = self.widget(tab)
+        if editorTab.isModified():
             txtSaveOnRemove = QCoreApplication.translate("PythonConsole",
                                                          "Python Console: Save File")
             txtMsgSaveOnRemove = QCoreApplication.translate("PythonConsole",
@@ -922,43 +752,32 @@ class EditorTabWidget(QTabWidget):
             res = QMessageBox.question(self, txtSaveOnRemove,
                                        txtMsgSaveOnRemove,
                                        QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel)
-            if res == QMessageBox.Save:
-                tabWidget.save()
-            elif res == QMessageBox.Cancel:
+            if res == QMessageBox.Cancel:
                 return
-            if tabWidget.path:
-                self.parent.updateTabListScript(tabWidget.path, action='remove')
+            if res == QMessageBox.Save:
+                editorTab.save()
+            if editorTab.path:
+                self.parent.updateTabListScript(editorTab.path, action='remove')
             self.removeTab(tab)
             if self.count() < 1:
                 self.newTabEditor()
         else:
-            if tabWidget.path:
-                self.parent.updateTabListScript(tabWidget.path, action='remove')
+            if editorTab.path:
+                self.parent.updateTabListScript(editorTab.path, action='remove')
             if self.count() <= 1:
                 self.removeTab(tab)
                 self.newTabEditor()
             else:
                 self.removeTab(tab)
-        self.currentWidget().newEditor.setFocus(Qt.TabFocusReason)
 
-    def buttonClosePressed(self):
-        self.closeCurrentWidget()
-
-    def closeCurrentWidget(self):
-        currWidget = self.currentWidget()
-        if currWidget and currWidget.close():
-            self.removeTab(self.currentIndex())
-            currWidget = self.currentWidget()
-            if currWidget:
-                currWidget.setFocus(Qt.TabFocusReason)
-        if currWidget.path in self.restoreTabList:
-            self.parent.updateTabListScript(currWidget.path, action='remove')
+        editorTab.deleteLater()
+        self.currentWidget()._editor.setFocus(Qt.TabFocusReason)
 
     def restoreTabsOrAddNew(self):
         """
         Restore tabs if they are found in the settings. If none are found it will add a new empty tab.
         """
-        # Restore script of the previuos session
+        # Restore scripts from the previous session
         tabScripts = self.settings.value("pythonConsole/tabScripts", [])
         self.restoreTabList = tabScripts
 
@@ -984,7 +803,7 @@ class EditorTabWidget(QTabWidget):
             self.newTabEditor(filename=None)
         self.topFrame.close()
         self.enableToolBarEditor(True)
-        self.currentWidget().newEditor.setFocus(Qt.TabFocusReason)
+        self.currentWidget()._editor.setFocus(Qt.TabFocusReason)
 
     def closeRestore(self):
         self.parent.updateTabListScript(None)
@@ -1080,10 +899,6 @@ class EditorTabWidget(QTabWidget):
                     iconWarning = QgsApplication.getThemeIcon("console/iconSyntaxErrorConsole.svg")
                     msgItem.setIcon(0, iconWarning)
                     self.parent.listClassMethod.addTopLevelItem(msgItem)
-                    # s = traceback.format_exc()
-                    # print('## Error: ')
-                    # sys.stderr.write(s)
-                    # pass
 
     def refreshSettingsEditor(self):
         objInspectorEnabled = self.settings.value("pythonConsole/enableObjectInsp",
