@@ -29,7 +29,12 @@ from qgis.PyQt.QtWidgets import QMainWindow, QApplication, QMenu, QTabWidget, QG
 from qgis.PyQt.QtGui import QIcon, QKeySequence
 
 from qgis.gui import QgsMessageBar
-from qgis.core import Qgis, QgsSettings, QgsMapLayer
+from qgis.core import (
+    Qgis,
+    QgsApplication,
+    QgsSettings,
+    QgsMapLayerType
+)
 from qgis.utils import OverrideCursor
 
 from .info_viewer import InfoViewer
@@ -56,6 +61,8 @@ class DBManager(QMainWindow):
         self.restoreState(settings.value("/DB_Manager/mainWindow/windowState", QByteArray(), type=QByteArray))
 
         self.toolBar.setIconSize(self.iface.iconSize())
+        self.toolBarOrientation()
+        self.toolBar.orientationChanged.connect(self.toolBarOrientation)
         self.tabs.currentChanged.connect(self.tabChanged)
         self.tree.selectedItemChanged.connect(self.itemChanged)
         self.tree.model().dataChanged.connect(self.iface.reloadConnections)
@@ -86,6 +93,8 @@ class DBManager(QMainWindow):
         with OverrideCursor(Qt.WaitCursor):
             try:
                 self.reloadButtons()
+                # Force-reload information on the layer
+                self.info.setDirty()
                 # clear preview, this will delete the layer in preview tab
                 self.preview.loadPreview(None)
                 self.refreshTabs()
@@ -168,7 +177,7 @@ class DBManager(QMainWindow):
             return
 
         inLayer = table.toMapLayer()
-        if inLayer.type() != QgsMapLayer.VectorLayer:
+        if inLayer.type() != QgsMapLayerType.VectorLayer:
             self.infoBar.pushMessage(
                 self.tr("Select a vector or a tabular layer you want export."),
                 Qgis.Warning, self.iface.messageTimeout())
@@ -212,7 +221,7 @@ class DBManager(QMainWindow):
     def update_query_tab_name(self, index, dbname, queryname):
         if not queryname:
             queryname = self.tr("Query")
-        tabname = u"%s (%s)" % (queryname, dbname)
+        tabname = "%s (%s)" % (queryname, dbname)
         self.tabs.setTabText(index, tabname)
 
     def showSystemTables(self):
@@ -353,8 +362,23 @@ class DBManager(QMainWindow):
     def close_tab(self, index):
         widget = self.tabs.widget(index)
         if widget not in [self.info, self.table, self.preview]:
-            self.tabs.removeTab(index)
-            widget.deleteLater()
+            if hasattr(widget, "close"):
+                if widget.close():
+                    self.tabs.removeTab(index)
+                    widget.deleteLater()
+            else:
+                self.tabs.removeTab(index)
+                widget.deleteLater()
+
+    def toolBarOrientation(self):
+        button_style = Qt.ToolButtonIconOnly
+        if self.toolBar.orientation() == Qt.Horizontal:
+            button_style = Qt.ToolButtonTextBesideIcon
+
+        widget = self.toolBar.widgetForAction(self.actionImport)
+        widget.setToolButtonStyle(button_style)
+        widget = self.toolBar.widgetForAction(self.actionExport)
+        widget.setToolButtonStyle(button_style)
 
     def setupUi(self):
         self.setWindowTitle(self.tr("DB Manager"))
@@ -393,7 +417,7 @@ class DBManager(QMainWindow):
         self.layout.addWidget(self.infoBar, 0, 0, 1, 1)
 
         # create database tree
-        self.dock = QDockWidget("Tree", self)
+        self.dock = QDockWidget(self.tr("Providers"), self)
         self.dock.setObjectName("DB_Manager_DBView")
         self.dock.setFeatures(QDockWidget.DockWidgetMovable)
         self.tree = DBTree(self)
@@ -418,7 +442,7 @@ class DBManager(QMainWindow):
         self.setMenuBar(self.menuBar)
 
         # create toolbar
-        self.toolBar = QToolBar("Default", self)
+        self.toolBar = QToolBar(self.tr("Default"), self)
         self.toolBar.setObjectName("DB_Manager_ToolBar")
         self.addToolBar(self.toolBar)
 
@@ -429,7 +453,7 @@ class DBManager(QMainWindow):
         sep.setObjectName("DB_Manager_DbMenu_placeholder")
         sep.setVisible(False)
 
-        self.actionRefresh = self.menuDb.addAction(QIcon(":/db_manager/actions/refresh"), self.tr("&Refresh"),
+        self.actionRefresh = self.menuDb.addAction(QgsApplication.getThemeIcon("/mActionRefresh.svg"), self.tr("&Refresh"),
                                                    self.refreshActionSlot, QKeySequence("F5"))
         self.actionSqlWindow = self.menuDb.addAction(QIcon(":/db_manager/actions/sql_window"), self.tr("&SQL Window"),
                                                      self.runSqlWindow, QKeySequence("F2"))
@@ -455,13 +479,14 @@ class DBManager(QMainWindow):
                                                      QApplication.translate("DBManager", "&Export to File…"),
                                                      self.exportActionSlot)
         self.menuTable.addSeparator()
-        #self.actionShowSystemTables = self.menuTable.addAction(self.tr("Show system tables/views"), self.showSystemTables)
-        #self.actionShowSystemTables.setCheckable(True)
-        #self.actionShowSystemTables.setChecked(True)
+        # self.actionShowSystemTables = self.menuTable.addAction(self.tr("Show system tables/views"), self.showSystemTables)
+        # self.actionShowSystemTables.setCheckable(True)
+        # self.actionShowSystemTables.setChecked(True)
         actionMenuTable.setVisible(False)
 
         # add actions to the toolbar
         self.toolBar.addAction(self.actionRefresh)
         self.toolBar.addAction(self.actionSqlWindow)
+        self.toolBar.addSeparator()
         self.toolBar.addAction(self.actionImport)
         self.toolBar.addAction(self.actionExport)
