@@ -70,7 +70,7 @@ class PostGisDBPlugin(DBPlugin):
     def connect(self, parent=None):
         conn_name = self.connectionName()
         settings = QgsSettings()
-        settings.beginGroup(u"/%s/%s" % (self.connectionSettingsKey(), conn_name))
+        settings.beginGroup("/%s/%s" % (self.connectionSettingsKey(), conn_name))
 
         if not settings.contains("database"):  # non-existent entry?
             raise InvalidDataException(self.tr('There is no defined database connection "{0}".').format(conn_name))
@@ -84,7 +84,7 @@ class PostGisDBPlugin(DBPlugin):
 
         useEstimatedMetadata = settings.value("estimatedMetadata", False, type=bool)
         try:
-            sslmode = settings.value("sslmode", QgsDataSourceUri.SslPrefer, type=int)
+            sslmode = settings.enumValue("sslmode", QgsDataSourceUri.SslPrefer)
         except TypeError:
             sslmode = QgsDataSourceUri.SslPrefer
 
@@ -228,7 +228,7 @@ class PGTable(Table):
             rule_name = parts[1]
             rule_action = parts[2]
 
-            msg = self.tr(u"Do you want to {0} rule {1}?").format(rule_action, rule_name)
+            msg = self.tr("Do you want to {0} rule {1}?").format(rule_action, rule_name)
 
             QApplication.restoreOverrideCursor()
 
@@ -308,6 +308,24 @@ class PGVectorTable(PGTable, VectorTable):
             return True
         return VectorTable.runAction(self, action)
 
+    def geometryType(self):
+        """ Returns the proper WKT type.
+        PostGIS records type like this:
+        | WKT Type     | geomType    | geomDim |
+        |--------------|-------------|---------|
+        | LineString   | LineString  | 2       |
+        | LineStringZ  | LineString  | 3       |
+        | LineStringM  | LineStringM | 3       |
+        | LineStringZM | LineString  | 4       |
+        """
+        geometryType = self.geomType
+        if self.geomDim == 3 and self.geomType[-1] != "M":
+            geometryType += "Z"
+        elif self.geomDim == 4:
+            geometryType += "ZM"
+
+        return geometryType
+
 
 class PGRasterTable(PGTable, RasterTable):
 
@@ -327,12 +345,12 @@ class PGRasterTable(PGTable, RasterTable):
 
         if not uri:
             uri = self.database().uri()
-        service = (u'service=\'%s\'' % uri.service()) if uri.service() else ''
-        dbname = (u'dbname=\'%s\'' % uri.database()) if uri.database() else ''
-        host = (u'host=%s' % uri.host()) if uri.host() else ''
-        user = (u'user=%s' % uri.username()) if uri.username() else ''
-        passw = (u'password=%s' % uri.password()) if uri.password() else ''
-        port = (u'port=%s' % uri.port()) if uri.port() else ''
+        service = ('service=\'%s\'' % uri.service()) if uri.service() else ''
+        dbname = ('dbname=\'%s\'' % uri.database()) if uri.database() else ''
+        host = ('host=%s' % uri.host()) if uri.host() else ''
+        user = ('user=%s' % uri.username()) if uri.username() else ''
+        passw = ('password=%s' % uri.password()) if uri.password() else ''
+        port = ('port=%s' % uri.port()) if uri.port() else ''
 
         schema = self.schemaName() if self.schemaName() else 'public'
         table = '"%s"."%s"' % (schema, self.name)
@@ -341,26 +359,26 @@ class PGRasterTable(PGTable, RasterTable):
             # postgresraster provider *requires* a dbname
             connector = self.database().connector
             r = connector._execute(None, "SELECT current_database()")
-            dbname = (u'dbname=\'%s\'' % connector._fetchone(r)[0])
+            dbname = ('dbname=\'%s\'' % connector._fetchone(r)[0])
             connector._close_cursor(r)
 
         # Find first raster field
         col = ''
         for fld in self.fields():
             if fld.dataType == "raster":
-                col = u'column=\'%s\'' % fld.name
+                col = 'column=\'%s\'' % fld.name
                 break
 
-        uri = u'%s %s %s %s %s %s %s table=%s' % \
+        uri = '%s %s %s %s %s %s %s table=%s' % \
             (service, dbname, host, user, passw, port, col, table)
 
         return uri
 
     def mimeUri(self):
-        uri = u"raster:postgresraster:{}:{}".format(self.name, re.sub(":", r"\:", self.uri()))
+        uri = "raster:postgresraster:{}:{}".format(self.name, re.sub(":", r"\:", self.uri()))
         return uri
 
-    def toMapLayer(self):
+    def toMapLayer(self, geometryType=None, crs=None):
         from qgis.core import QgsRasterLayer, QgsContrastEnhancement, QgsDataSourceUri, QgsCredentials
 
         rl = QgsRasterLayer(self.uri(), self.name, "postgresraster")
@@ -414,14 +432,14 @@ class PGTableField(TableField):
         sql_cpt = "Select count(*) from pg_description pd, pg_class pc, pg_attribute pa where relname = '%s' and attname = '%s' and pa.attrelid = pc.oid and pd.objoid = pc.oid and pd.objsubid = pa.attnum" % (tab.name, self.name)
         # SQL Query that return the comment of the field
         sql = "Select pd.description from pg_description pd, pg_class pc, pg_attribute pa where relname = '%s' and attname = '%s' and pa.attrelid = pc.oid and pd.objoid = pc.oid and pd.objsubid = pa.attnum" % (tab.name, self.name)
-        c = tab.database().connector._execute(None, sql_cpt) # Execute Check query
-        res = tab.database().connector._fetchone(c)[0] # Store result
+        c = tab.database().connector._execute(None, sql_cpt)  # Execute Check query
+        res = tab.database().connector._fetchone(c)[0]  # Store result
         if res == 1:
             # When a comment exists
-            c = tab.database().connector._execute(None, sql) # Execute query
-            res = tab.database().connector._fetchone(c)[0] # Store result
-            tab.database().connector._close_cursor(c) # Close cursor
-            return res # Return comment
+            c = tab.database().connector._execute(None, sql)  # Execute query
+            res = tab.database().connector._fetchone(c)[0]  # Store result
+            tab.database().connector._close_cursor(c)  # Close cursor
+            return res  # Return comment
         else:
             return ''
 
