@@ -38,10 +38,12 @@ from qgis.core import (QgsMessageLog,
                        QgsProcessingParameterDefinition,
                        QgsProcessingOutputVectorLayer,
                        QgsProcessingOutputRasterLayer,
+                       QgsProcessingOutputPointCloudLayer,
                        QgsProcessingOutputMapLayer,
                        QgsProcessingOutputMultipleLayers,
                        QgsProcessingFeedback,
                        QgsRuntimeProfiler)
+from qgis.analysis import QgsNativeAlgorithms
 
 import processing
 from processing.core.ProcessingConfig import ProcessingConfig
@@ -55,24 +57,12 @@ from processing.tools import dataobjects
 with QgsRuntimeProfiler.profile('Import QGIS Provider'):
     from processing.algs.qgis.QgisAlgorithmProvider import QgisAlgorithmProvider  # NOQA
 
-with QgsRuntimeProfiler.profile('Import GRASS Provider'):
-    from processing.algs.grass7.Grass7AlgorithmProvider import Grass7AlgorithmProvider
-
 with QgsRuntimeProfiler.profile('Import GDAL Provider'):
     from processing.algs.gdal.GdalAlgorithmProvider import GdalAlgorithmProvider  # NOQA
 
-with QgsRuntimeProfiler.profile('Import OTB Provider'):
-    from processing.algs.otb.OtbAlgorithmProvider import OtbAlgorithmProvider  # NOQA
-
-with QgsRuntimeProfiler.profile('Import SAGA Provider'):
-    from processing.algs.saga.SagaAlgorithmProvider import SagaAlgorithmProvider  # NOQA
-
 with QgsRuntimeProfiler.profile('Import Script Provider'):
     from processing.script.ScriptAlgorithmProvider import ScriptAlgorithmProvider  # NOQA
 
-
-# from processing.preconfigured.PreconfiguredAlgorithmProvider import PreconfiguredAlgorithmProvider  # NOQA
-
 # should be loaded last - ensures that all dependent algorithms are available when loading models
 from processing.modeler.ModelerAlgorithmProvider import ModelerAlgorithmProvider  # NOQA
 from processing.modeler.ProjectProvider import ProjectProvider  # NOQA
@@ -99,13 +89,24 @@ class Processing(object):
             return
 
         with QgsRuntimeProfiler.profile('Initialize'):
+
+            # add native provider if not already added
+            if "native" not in [p.id() for p in QgsApplication.processingRegistry().providers()]:
+                QgsApplication.processingRegistry().addProvider(QgsNativeAlgorithms(QgsApplication.processingRegistry()))
+
+            # add 3d provider if available and not already added
+            if "3d" not in [p.id() for p in QgsApplication.processingRegistry().providers()]:
+                try:
+                    from qgis._3d import Qgs3DAlgorithms
+                    QgsApplication.processingRegistry().addProvider(Qgs3DAlgorithms(QgsApplication.processingRegistry()))
+                except ImportError:
+                    # no 3d library available
+                    pass
+
             # Add the basic providers
             for c in [
                 QgisAlgorithmProvider,
-                Grass7AlgorithmProvider,
                 GdalAlgorithmProvider,
-                OtbAlgorithmProvider,
-                SagaAlgorithmProvider,
                 ScriptAlgorithmProvider,
                 ModelerAlgorithmProvider,
                 ProjectProvider
@@ -113,6 +114,25 @@ class Processing(object):
                 p = c()
                 if QgsApplication.processingRegistry().addProvider(p):
                     Processing.BASIC_PROVIDERS.append(p)
+
+            if QgsApplication.platform() == 'external':
+                # for external applications we must also load the builtin providers stored in separate plugins
+                try:
+                    from grassprovider.Grass7AlgorithmProvider import Grass7AlgorithmProvider
+                    p = Grass7AlgorithmProvider()
+                    if QgsApplication.processingRegistry().addProvider(p):
+                        Processing.BASIC_PROVIDERS.append(p)
+                except ImportError:
+                    pass
+
+                try:
+                    from otbprovider.OtbAlgorithmProvider import OtbAlgorithmProvider
+                    p = OtbAlgorithmProvider()
+                    if QgsApplication.processingRegistry().addProvider(p):
+                        Processing.BASIC_PROVIDERS.append(p)
+                except ImportError:
+                    pass
+
             # And initialize
             ProcessingConfig.initialize()
             ProcessingConfig.readSettings()
@@ -169,7 +189,7 @@ class Processing(object):
                     if out.name() not in results:
                         continue
 
-                    if isinstance(out, (QgsProcessingOutputVectorLayer, QgsProcessingOutputRasterLayer, QgsProcessingOutputMapLayer)):
+                    if isinstance(out, (QgsProcessingOutputVectorLayer, QgsProcessingOutputRasterLayer, QgsProcessingOutputPointCloudLayer, QgsProcessingOutputMapLayer)):
                         result = results[out.name()]
                         if not isinstance(result, QgsMapLayer):
                             layer = context.takeResultLayer(result)  # transfer layer ownership out of context
@@ -180,7 +200,7 @@ class Processing(object):
                         if result:
                             layers_result = []
                             for l in result:
-                                if not isinstance(result, QgsMapLayer):
+                                if not isinstance(l, QgsMapLayer):
                                     layer = context.takeResultLayer(l)  # transfer layer ownership out of context
                                     if layer:
                                         layers_result.append(layer)
