@@ -21,19 +21,14 @@ __author__ = 'Victor Olaya'
 __date__ = 'April 2014'
 __copyright__ = '(C) 201, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
-
 import os
 import shutil
 from qgis.PyQt.QtWidgets import QFileDialog, QMessageBox
-from qgis.PyQt.QtCore import QFileInfo, QCoreApplication
+from qgis.PyQt.QtCore import QFileInfo, QCoreApplication, QDir
 
 from qgis.core import QgsApplication, QgsSettings, QgsProcessingModelAlgorithm
 
 from processing.gui.ToolboxAction import ToolboxAction
-from processing.modeler.exceptions import WrongModelException
 from processing.modeler.ModelerUtils import ModelerUtils
 
 pluginPath = os.path.split(os.path.dirname(__file__))[0]
@@ -50,7 +45,7 @@ class AddModelFromFileAction(ToolboxAction):
 
     def execute(self):
         settings = QgsSettings()
-        lastDir = settings.value('Processing/lastModelsDir', '')
+        lastDir = settings.value('Processing/lastModelsDir', QDir.homePath())
         filename, selected_filter = QFileDialog.getOpenFileName(self.toolbox,
                                                                 self.tr('Open Model', 'AddModelFromFileAction'), lastDir,
                                                                 self.tr('Processing models (*.model3 *.MODEL3)', 'AddModelFromFileAction'))
@@ -65,6 +60,25 @@ class AddModelFromFileAction(ToolboxAction):
                     self.tr('Open Model', 'AddModelFromFileAction'),
                     self.tr('The selected file does not contain a valid model', 'AddModelFromFileAction'))
                 return
+
+            if QgsApplication.instance().processingRegistry().algorithmById('model:{}'.format(alg.id())):
+                QMessageBox.warning(
+                    self.toolbox,
+                    self.tr('Open Model', 'AddModelFromFileAction'),
+                    self.tr('Model with the same name already exists', 'AddModelFromFileAction'))
+                return
+
             destFilename = os.path.join(ModelerUtils.modelsFolders()[0], os.path.basename(filename))
+            if os.path.exists(destFilename):
+                reply = QMessageBox.question(
+                    self.toolbox,
+                    self.tr('Open Model', 'AddModelFromFileAction'),
+                    self.tr('There is already a model file with the same name. Overwrite?', 'AddModelFromFileAction'),
+                    QMessageBox.Yes | QMessageBox.No,
+                    QMessageBox.No)
+
+                if reply == QMessageBox.No:
+                    return
+
             shutil.copyfile(filename, destFilename)
             QgsApplication.processingRegistry().providerById('model').refreshAlgorithms()
