@@ -21,24 +21,18 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '8a899c8758b30adc3f8d6df682a5702dd3bfe6f0'
-
 import os
 
-from qgis.PyQt.QtXml import QDomDocument
-
 from qgis.core import (Qgis,
                        QgsApplication,
                        QgsProcessingProvider,
                        QgsMessageLog,
                        QgsProcessingModelAlgorithm,
-                       QgsProcessingUtils,
-                       QgsXmlUtils)
+                       QgsRuntimeProfiler)
 
 from processing.core.ProcessingConfig import ProcessingConfig, Setting
 
+from processing.gui.ContextAction import ContextAction
 from processing.gui.ProviderActions import (ProviderActions,
                                             ProviderContextMenuActions)
 
@@ -46,8 +40,8 @@ from processing.modeler.AddModelFromFileAction import AddModelFromFileAction
 from processing.modeler.CreateNewModelAction import CreateNewModelAction
 from processing.modeler.DeleteModelAction import DeleteModelAction
 from processing.modeler.EditModelAction import EditModelAction
+from processing.modeler.ExportModelAsPythonScriptAction import ExportModelAsPythonScriptAction
 from processing.modeler.OpenModelFromFileAction import OpenModelFromFileAction
-from processing.modeler.WrongModelException import WrongModelException
 from processing.modeler.ModelerUtils import ModelerUtils
 
 pluginPath = os.path.split(os.path.dirname(__file__))[0]
@@ -58,8 +52,11 @@ class ModelerAlgorithmProvider(QgsProcessingProvider):
     def __init__(self):
         super().__init__()
         self.actions = [CreateNewModelAction(), OpenModelFromFileAction(), AddModelFromFileAction()]
-        self.contextMenuActions = [EditModelAction(), DeleteModelAction()]
+        sep_action = ContextAction()
+        sep_action.is_separator = True
+        self.contextMenuActions = [EditModelAction(), DeleteModelAction(), sep_action, ExportModelAsPythonScriptAction()]
         self.algs = []
+        self.isLoading = False
 
         # must reload models if providers list is changed - previously unavailable algorithms
         # which models depend on may now be available
@@ -69,14 +66,16 @@ class ModelerAlgorithmProvider(QgsProcessingProvider):
         self.refreshAlgorithms()
 
     def load(self):
-        ProcessingConfig.settingIcons[self.name()] = self.icon()
-        ProcessingConfig.addSetting(Setting(self.name(),
-                                            ModelerUtils.MODELS_FOLDER, self.tr('Models folder', 'ModelerAlgorithmProvider'),
-                                            ModelerUtils.defaultModelsFolder(), valuetype=Setting.MULTIPLE_FOLDERS))
-        ProviderActions.registerProviderActions(self, self.actions)
-        ProviderContextMenuActions.registerProviderContextMenuActions(self.contextMenuActions)
-        ProcessingConfig.readSettings()
-        self.refreshAlgorithms()
+        with QgsRuntimeProfiler.profile('Model Provider'):
+            ProcessingConfig.settingIcons[self.name()] = self.icon()
+            ProcessingConfig.addSetting(Setting(self.name(),
+                                                ModelerUtils.MODELS_FOLDER, self.tr('Models folder', 'ModelerAlgorithmProvider'),
+                                                ModelerUtils.defaultModelsFolder(), valuetype=Setting.MULTIPLE_FOLDERS))
+            ProviderActions.registerProviderActions(self, self.actions)
+            ProviderContextMenuActions.registerProviderContextMenuActions(self.contextMenuActions)
+            ProcessingConfig.readSettings()
+            self.refreshAlgorithms()
+
         return True
 
     def unload(self):
@@ -102,12 +101,17 @@ class ModelerAlgorithmProvider(QgsProcessingProvider):
         return True
 
     def loadAlgorithms(self):
-        self.algs = []
-        folders = ModelerUtils.modelsFolders()
-        for f in folders:
-            self.loadFromFolder(f)
-        for a in self.algs:
-            self.addAlgorithm(a)
+        with QgsRuntimeProfiler.profile('Load model algorithms'):
+            if self.isLoading:
+                return
+            self.isLoading = True
+            self.algs = []
+            folders = ModelerUtils.modelsFolders()
+            for f in folders:
+                self.loadFromFolder(f)
+            for a in self.algs:
+                self.addAlgorithm(a)
+            self.isLoading = False
 
     def loadFromFolder(self, folder):
         if not os.path.exists(folder):
@@ -115,17 +119,13 @@ class ModelerAlgorithmProvider(QgsProcessingProvider):
         for path, subdirs, files in os.walk(folder):
             for descriptionFile in files:
                 if descriptionFile.endswith('model3'):
-                    try:
-                        fullpath = os.path.join(path, descriptionFile)
+                    fullpath = os.path.join(path, descriptionFile)
 
-                        alg = QgsProcessingModelAlgorithm()
-                        if alg.fromFile(fullpath):
-                            if alg.name():
-                                alg.setSourceFilePath(fullpath)
-                                self.algs.append(alg)
-                        else:
-                            QgsMessageLog.logMessage(self.tr('Could not load model {0}', 'ModelerAlgorithmProvider').format(descriptionFile),
-                                                     self.tr('Processing'), Qgis.Critical)
-                    except WrongModelException as e:
-                        QgsMessageLog.logMessage(self.tr('Could not load model {0}\n{1}', 'ModelerAlgorithmProvider').format(descriptionFile, e.msg),
+                    alg = QgsProcessingModelAlgorithm()
+                    if alg.fromFile(fullpath):
+                        if alg.name():
+                            alg.setSourceFilePath(fullpath)
+                            self.algs.append(alg)
+                    else:
+                        QgsMessageLog.logMessage(self.tr('Could not load model {0}', 'ModelerAlgorithmProvider').format(descriptionFile),
                                                  self.tr('Processing'), Qgis.Critical)
